@@ -37,11 +37,14 @@ int viewer_session_set_trace_chunk_copy(struct relay_viewer_session *vsession,
 	int ret = 0;
 	struct lttng_trace_chunk *viewer_chunk;
 
-	assert(relay_session_trace_chunk);
 	lttng_trace_chunk_put(vsession->current_trace_chunk);
 	vsession->current_trace_chunk = NULL;
 
 	DBG("Copying relay session's current trace chunk to the viewer session");
+	if (!relay_session_trace_chunk) {
+		goto end;
+	}
+
 	viewer_chunk = lttng_trace_chunk_copy(relay_session_trace_chunk);
 	if (!viewer_chunk) {
 		ERR("Failed to create a viewer trace chunk from the relay session's current chunk");
@@ -74,7 +77,6 @@ enum lttng_viewer_attach_return_code viewer_session_attach(
 	} else {
 		int ret;
 
-		assert(session->current_trace_chunk);
 		assert(!vsession->current_trace_chunk);
 		session->viewer_attached = true;
 
