@@ -21,6 +21,7 @@
 
 static void viewer_stream_destroy(struct relay_viewer_stream *vstream)
 {
+	lttng_trace_chunk_put(vstream->stream_file.trace_chunk);
 	free(vstream->path_name);
 	free(vstream->channel_name);
 	free(vstream);
@@ -40,12 +41,8 @@ struct relay_viewer_stream *viewer_stream_create(struct relay_stream *stream,
 		enum lttng_viewer_seek seek_t)
 {
 	struct relay_viewer_stream *vstream = NULL;
-	const bool acquired_reference = lttng_trace_chunk_get(trace_chunk);
 
 	ASSERT_LOCKED(stream->lock);
-	if (!acquired_reference) {
-		goto error;
-	}
 
 	vstream = zmalloc(sizeof(*vstream));
 	if (!vstream) {
@@ -53,8 +50,14 @@ struct relay_viewer_stream *viewer_stream_create(struct relay_stream *stream,
 		goto error;
 	}
 
+	if (trace_chunk) {
+		const bool acquired_reference = lttng_trace_chunk_get(
+				trace_chunk);
+
+		assert(acquired_reference);
+	}
+
 	vstream->stream_file.trace_chunk = trace_chunk;
-	trace_chunk = NULL;
 	vstream->path_name = lttng_strndup(stream->path_name, LTTNG_VIEWER_PATH_MAX);
 	if (vstream->path_name == NULL) {
 		PERROR("relay viewer path_name alloc");
@@ -119,7 +122,7 @@ struct relay_viewer_stream *viewer_stream_create(struct relay_stream *stream,
 	 */
 	if (stream->index_file == NULL) {
 		vstream->index_file = NULL;
-	} else {
+	} else if (vstream->stream_file.trace_chunk) {
 		const uint32_t connection_major = stream->trace->session->major;
 		const uint32_t connection_minor = stream->trace->session->minor;
 		enum lttng_trace_chunk_status chunk_status;
@@ -147,7 +150,7 @@ struct relay_viewer_stream *viewer_stream_create(struct relay_stream *stream,
 	 * If we never received a data file for the current stream, delay the
 	 * opening, otherwise open it right now.
 	 */
-	if (stream->file) {
+	if (stream->file && vstream->stream_file.trace_chunk) {
 		int ret;
 		char file_path[LTTNG_PATH_MAX];
 		enum lttng_trace_chunk_status status;
@@ -183,6 +186,8 @@ struct relay_viewer_stream *viewer_stream_create(struct relay_stream *stream,
 				vstream);
 	}
 
+	vstream->last_seen_rotation_count = stream->completed_rotation_count;
+
 	/* Globally visible after the add unique. */
 	lttng_ht_node_init_u64(&vstream->stream_n, stream->stream_handle);
 	urcu_ref_init(&vstream->ref);
@@ -194,9 +199,6 @@ error:
 	if (vstream) {
 		viewer_stream_destroy(vstream);
 	}
-	if (trace_chunk && acquired_reference) {
-		lttng_trace_chunk_put(trace_chunk);
-	}
 	return NULL;
 }
 
