@@ -14,6 +14,7 @@
 #include <unistd.h>
 
 #include <common/macros.h>
+#include <common/sessiond-comm/sessiond-comm.h>
 
 /*
  * The run-as process is launched by forking without an exec*() call. This means
@@ -70,6 +71,10 @@ int run_as_extract_sdt_probe_offsets(int fd, const char *provider_name,
 		const char* probe_name, uid_t uid, gid_t gid,
 		uint64_t **offsets, uint32_t *num_offset);
 LTTNG_HIDDEN
+int run_as_generate_filter_bytecode(const char *filter_expression,
+		const struct lttng_credentials *creds,
+		struct lttng_bytecode **bytecode);
+LTTNG_HIDDEN
 int run_as_create_worker(const char *procname,
 		post_fork_cleanup_cb clean_up_func, void *clean_up_user_data);
 LTTNG_HIDDEN
