@@ -15,10 +15,10 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <errno.h>
 #include <inttypes.h>
 
 #include <common/common.h>
+#include <common/compat/errno.h>
 
 #include "sessiond-comm.h"
 
